searchData={"items":[{"type":"module","title":"PhoenixTest","doc":"PhoenixTest provides a unified way of writing feature tests -- regardless of\n  whether you're testing LiveView pages or static pages.\n\n  It also handles navigation between LiveView and static pages seamlessly. So, you\n  don't have to worry about what type of page you're visiting. Just write the\n  tests from the user's perspective.\n\n  Thus, you can test a flow going from static to LiveView pages and back without\n  having to worry about the underlying implementation.\n\n  This is a sample flow:\n\n  ```elixir\n  test \"admin can create a user\", %{conn: conn} do\n    conn\n    |> visit(\"/\")\n    |> click_link(\"Users\")\n    |> fill_form(\"#user-form\", name: \"Aragorn\", email: \"aragorn@dunedan.com\")\n    |> click_button(\"Create\")\n    |> assert_has(\".user\", \"Aragorn\")\n  end\n  ```\n\n  Note that PhoenixTest does _not_ handle JavaScript. If you're looking for\n  something that supports JavaScript, take a look at\n  [Wallaby](https://hexdocs.pm/wallaby/readme.html).","ref":"PhoenixTest.html"},{"type":"function","title":"PhoenixTest.assert_has/3","doc":"Assert helper to ensure an element with given CSS selector and `text` is\npresent.\n\nIt'll raise an error if no elements are found, but it will not raise if more\nthan one matching element is found.","ref":"PhoenixTest.html#assert_has/3"},{"type":"function","title":"PhoenixTest.click_button/2","doc":"Perfoms action defined by button.\n\n- If the button has a `phx-click` on it, it'll send the event to the LiveView.\n\n- If the button doesn't have a `phx-click` on it, it'll submit the parent form.\n\nThis function can be preceded by `fill_form` to fill out a form and\nsubsequently submit the form. Note that `fill_form/3` + `click_button/2` works\nfor both static and live pages.\n\nIf `click_button/2` is used alone (without a `phx-click`), it is assumed it is a\nform with a single button (e.g. \"Delete\").","ref":"PhoenixTest.html#click_button/2"},{"type":"function","title":"Examples - PhoenixTest.click_button/2","doc":"```elixir\n# form with single button or button with `phx-click`\nsession\n|> click_button(\"Delete\")\n\n# fill out form and then submit\nsession\n|> fill_form(\"#user-form\", name: \"Aragorn\")\n|> click_button(\"Create\")\n```","ref":"PhoenixTest.html#click_button/2-examples"},{"type":"function","title":"PhoenixTest.click_button/3","doc":"Performs action defined by button with CSS selector.\n\nSee `click_button/2` for more details.","ref":"PhoenixTest.html#click_button/3"},{"type":"function","title":"PhoenixTest.click_link/2","doc":"Clicks a link with given text and performs the action.\n\nHere's how it handles different types of `a` tags:\n\n- With `href`: follows it to the next page\n- With `phx-submit`: it'll send the event to the appropriate LiveView\n- With live redirect: it'll follow the live navigation to the next LiveView\n- With live patch: it'll patch the current LiveView","ref":"PhoenixTest.html#click_link/2"},{"type":"function","title":"PhoenixTest.click_link/3","doc":"Clicks a link with given CSS selector and text and performs the action.\nselector to target the link.\n\nSee `click_link/2` for more details.","ref":"PhoenixTest.html#click_link/3"},{"type":"function","title":"PhoenixTest.fill_form/3","doc":"Fills form data, validating that input fields are present.\n\nThis can be used by both static and live pages.\n\nIf it is a LiveView, and if the form has a `phx-change` attribute defined,\n`fill_form/3` will trigger the `phx-change` event.\n\nThis can be followed by a `click_button/3` to submit the form.","ref":"PhoenixTest.html#fill_form/3"},{"type":"function","title":"PhoenixTest.refute_has/3","doc":"Opposite of `assert_has/3` helper. Verifies that element with\ngiven CSS selector and `text` is _not_ present.\n\nIt'll raise an error if an element that matches selector and text is  found.","ref":"PhoenixTest.html#refute_has/3"},{"type":"function","title":"PhoenixTest.submit_form/3","doc":"Submits form in the same way one would do by pressing ` `.\n\nNote that this does not validate presence of the submit button.","ref":"PhoenixTest.html#submit_form/3"},{"type":"function","title":"Examples - PhoenixTest.submit_form/3","doc":"```elixir\nsession\n|> submit_form(\"#user-form\", name: \"Aragorn\")\n```","ref":"PhoenixTest.html#submit_form/3-examples"},{"type":"function","title":"PhoenixTest.visit/2","doc":"Entrypoint to create a session.\n\n`visit/2` takes a `Plug.Conn` struct and the path to visit.\n\nIt returns a `session` which the rest of the `PhoenixTest` functions can use.\n\nNote that `visit/2` is smart enough to know if the page you're visiting is a\nLiveView or a static view. You don't need to worry about which type of page\nyou're visiting.","ref":"PhoenixTest.html#visit/2"},{"type":"extras","title":"Introduction","doc":"# PhoenixTest\n\n[![Module Version](https://img.shields.io/hexpm/v/phoenix_test.svg)](https://hex.pm/packages/phoenix_test/)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/phoenix_test/)\n[![License](https://img.shields.io/hexpm/l/phoenix_test.svg)](https://github.com/germsvel/phoenix_test/blob/main/LICENSE)\n\nPhoenixTest provides a unified way of writing feature tests -- regardless of\nwhether you're testing LiveView pages or static pages.\n\nIt also handles navigation between LiveView and static pages seamlessly. So, you\ndon't have to worry about what type of page you're visiting. Just write the\ntests from the user's perspective.\n\nThus, you can test a flow going from static to LiveView pages and back without\nhaving to worry about the underlying implementation.\n\nThis is a sample flow:\n\n```elixir\ntest \"admin can create a user\", %{conn: conn} do\n  conn\n  |> visit(\"/\")\n  |> click_link(\"Users\")\n  |> fill_form(\"#user-form\", name: \"Aragorn\", email: \"aragorn@dunedain.com\")\n  |> click_button(\"Create\")\n  |> assert_has(\".user\", \"Aragorn\")\nend\n```\n\nNote that PhoenixTest does not handle JavaScript. If you're looking for\nsomething that supports JavaScript, take a look at\n[Wallaby](https://hexdocs.pm/wallaby/readme.html).","ref":"readme.html"},{"type":"extras","title":"Setup - Introduction","doc":"PhoenixTest requires Phoenix `1.7+` and LiveView `0.20+`. It may work with\nearlier versions, but I have not tested that.\n\n#","ref":"readme.html#setup"},{"type":"extras","title":"Installation - Introduction","doc":"Add `phoenix_test` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:phoenix_test, \"~> 0.1.0\", only: :test, runtime: false}\n  ]\nend\n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Introduction","doc":"In `config/test.exs` specify the endpoint to be used for routing requests:\n\n```elixir\nconfig :phoenix_test, :endpoint, MyAppWeb.Endpoint\n```\n\n#","ref":"readme.html#configuration"},{"type":"extras","title":"Adding a `FeatureCase` - Introduction","doc":"`PhoenixTest` helpers can be included via `import PhoenixTest`.\n\nBut since each test needs a `conn` struct to get started, you'll likely want to\nset up a few things before that.\n\nTo make that easier, it's helpful to create a `FeatureCase` module that can be\nused from your tests (replace `MyApp` with your app's name):\n\n```elixir\ndefmodule MyAppWeb.FeatureCase do\n  @moduledoc \"\"\"\n  This module defines the test case to be used by tests that require setting up\n  a connection to test feature tests.\n\n  Such tests rely on `PhoenixTest` and also import other functionality to\n  make it easier to build common data structures and interact with pages.\n\n  Finally, if the test case interacts with the database, we enable the SQL\n  sandbox, so changes done to the database are reverted at the end of every\n  test. If you are using PostgreSQL, you can even run database tests\n  asynchronously by setting `use MyAppWeb.FeatureCase, async: true`, although\n  this option is not recommended for other databases.\n  \"\"\"\n\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use MyAppWeb, :verified_routes\n\n      import MyAppWeb.FeatureCase\n\n      import PhoenixTest\n    end\n  end\n\n  setup tags do\n    pid = Ecto.Adapters.SQL.Sandbox.start_owner!(MyApp.Repo, shared: not tags[:async])\n    on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n\n    {:ok, conn: Phoenix.ConnTest.build_conn()}\n  end\nend\n```\n\nNote that we assume your Phoenix project is using Ecto and its phenomenal\n`SQL.Sandbox`. If it doesn't, feel free to remove the `SQL.Sandbox` code above.","ref":"readme.html#adding-a-featurecase"},{"type":"extras","title":"Usage - Introduction","doc":"Now, you can create your tests like this:\n\n```elixir\n# test/my_app_web/features/admin_can_create_user_test.exs\n\ndefmodule MyAppWeb.AdminCanCreateUserTest do\n  use MyAppWeb.FeatureCase, async: true\n\n  test \"admin can create user\", %{conn: conn} do\n    conn\n    |> visit(\"/\")\n    |> click_link(\"Users\")\n    |> fill_form(\"#user-form\", name: \"Aragorn\", email: \"aragorn@dunedain.com\")\n    |> click_button(\"Create\")\n    |> assert_has(\".user\", \"Aragorn\")\n  end\n```\n\nFor full documentation, take a look at [PhoenixTest module docs](https://hexdocs.pm/phoenix_test/PhoenixTest.html).","ref":"readme.html#usage"},{"type":"extras","title":"Why PhoenixTest? - Introduction","doc":"#","ref":"readme.html#why-phoenixtest"},{"type":"extras","title":"A unified way of writing feature tests - Introduction","doc":"With the advent of LiveView, I find myself writing less and less JavaScript.\n\nSure, there are sprinkles of it here and there, and there’s always the\noccasional need for something more.\n\nBut for the most part:\n\n- If I’m going to build a page that needs interactivity, I use LiveView.\n- If I’m going to write a static page, I use regular controllers + views/HTML\n  modules.\n\nThe problem is that LiveView pages and static pages have _vastly different_\ntesting strategies.\n\nIf we use LiveView, we have a good set of helpers.\n\n```elixir\n{:ok, view, _html} = live(conn, ~p\"/\")\n\nhtml =\n  view\n  |> element(\"#greet-guest\")\n  |> render_click()\n\nassert html =~ \"Hello, guest!\"\n```\n\nBut if we're testing a static page, we have to resort to controller testing:\n\n```elixir\nconn = get(conn, ~p\"/greet_page\")\n\nassert html_response(conn, 200) =~ \"Hello, guest!\"\n```\n\nThat means we don’t have ways of interacting with static pages at all!\n\nWhat if we want to submit a form or click a link? And what if a click takes us\nfrom a LiveView to a static view or vice versa?\n\nInstead, I'd like to have a unified way of testing Phoenix apps -- regardless of\nwhether we're testing LiveView pages or static pages.\n\n#","ref":"readme.html#a-unified-way-of-writing-feature-tests"},{"type":"extras","title":"Improved assertions - Introduction","doc":"And then there's the problem of assertions.\n\nBecause LiveView and controller tests use `=~` for assertions, the error\nmessages aren't very helpful when assertions fail.\n\nAfter all, we’re just comparing two blobs of text – and trust me, HTML pages can\nget very large and hard to read as a blob of text in your terminal.\n\nLiveView tries to help with its `has_element?/3` helper, which allows us to\ntarget elements by CSS selectors and text.\n\nUnfortunately, it still doesn't provide the best errors.\n\n`has_element?/3` only tells us what was passed into the function. It doesn't\ngive us a clue as to what else might've been on the page – maybe we just made a\nsmall typo and we have no idea!\n\nAnd that's where `PhoenixTest` comes in! A unified way of writing feature tests\nand improved assertions where they're needed!","ref":"readme.html#improved-assertions"},{"type":"extras","title":"Sponsors - Introduction","doc":"Made possible thanks to:\n\n \n   \n     \n     Elixir Streams","ref":"readme.html#sponsors"},{"type":"extras","title":"Changelog","doc":"Changelog\n=========\n\nNoteworthy changes are included here. For a full version of changes, see git\nhistory.\n\nTo see dates a version was published, see the [hex package\npage](https://hex.pm/packages/phoenix_test)","ref":"changelog.html"},{"type":"extras","title":"0.2.2 - Changelog","doc":"#","ref":"changelog.html#0-2-2"},{"type":"extras","title":"Added - Changelog","doc":"- Raise `AssertionError` instead of `RuntimeError` in assertions for more\n  consistent ExUnit error messages. Commit\n  [117bc59](https://github.com/germsvel/phoenix_test/commit/117bc59)\n- Update `fill_form` to handle that aren't direct children of the `form`\n  element. Commit\n  [46d6229](https://github.com/germsvel/phoenix_test/commit/46d6229)","ref":"changelog.html#added"},{"type":"extras","title":"0.2.1 - Changelog","doc":"#","ref":"changelog.html#0-2-1"},{"type":"extras","title":"Added - Changelog","doc":"- Improve printing of complex nested content in assertions. Commit\n  [7151834](https://github.com/germsvel/phoenix_test/commit/7151834)\n- Better error messages in forms when multiple submit buttons/inputs are found.\n  Commit [82492c6](https://github.com/germsvel/phoenix_test/commit/82492c6)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Allow using `refute_has` in pipes in the same way `assert_has` already works.\n  Commit [0484979](https://github.com/germsvel/phoenix_test/commit/0484979)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.0 - Changelog","doc":"#","ref":"changelog.html#0-2-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Update our static implementation to raise when we find many elements. That\n  brings it in line with how our LiveView implementation works. Commit\n  [daa4dca](https://github.com/germsvel/phoenix_test/commit/daa4dca)\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improved - Changelog","doc":"- Improves error messages in assertions. Commit [c995fc1](https://github.com/germsvel/phoenix_test/commit/c995fc1)","ref":"changelog.html#improved"},{"type":"extras","title":"0.1.1 - Changelog","doc":"#","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `click_link/3` and `click_button/3` which allow for specifying a CSS\n  selector. Commit [c7401b6](https://github.com/germsvel/phoenix_test/commit/c7401b6).","ref":"changelog.html#added"},{"type":"extras","title":"0.1.0 - Changelog","doc":"- Initial version of the library.","ref":"changelog.html#0-1-0"}],"content_type":"text/markdown"}